#include <windows.h>
#include <stdio.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "../include/filter.h"

///////////////////////////////////////////////////////////////////////////

int tutorialRunProc(const FilterActivation *fa, const FilterFunctions *ff);
int tutorialStartProc(FilterActivation *fa, const FilterFunctions *ff);
int tutorialEndProc(FilterActivation *fa, const FilterFunctions *ff);
long tutorialParamProc(FilterActivation *fa, const FilterFunctions *ff);
int tutorialConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void tutorialStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void tutorialScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool tutorialFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);

///////////////////////////////////////////////////////////////////////////

typedef struct MyFilterData {
    int lines;
	int doubleLines;
	float alpha;
    bool     fExpand;
    bool     fThird;
} MyFilterData;

ScriptFunctionDef tutorial_func_defs[]={
    { (ScriptFunctionPtr)tutorialScriptConfig, "Config", "0ii" },
    { NULL },
};

CScriptObject tutorial_obj={
    NULL, tutorial_func_defs
};

struct FilterDefinition filterDef_tutorial = {

    NULL, NULL, NULL,       // next, prev, module
    "a_megafilter",             // name
    "Emphasizes green and de-emphasizes blue in the image.",
                            // desc
    "video_mdu",               // maker
    NULL,                   // private_data
    sizeof(MyFilterData),   // inst_data_size

    NULL,                   // initProc
    NULL,                   // deinitProc
    tutorialRunProc,        // runProc
    tutorialParamProc,      // paramProc
    NULL,					//tutorialConfigProc,     // configProc
    tutorialStringProc,     // stringProc
    tutorialStartProc,      // startProc
    tutorialEndProc,        // endProc

    &tutorial_obj,          // script_obj
    tutorialFssProc,        // fssProc

};

///////////////////////////////////////////////////////////////////////////

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_tutorial;

///////////////////////////////////////////////////////////////////////////

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_tutorial = ff->addFilter(fm, &filterDef_tutorial, sizeof(FilterDefinition))))
        return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_tutorial);
}

///////////////////////////////////////////////////////////////////////////


int tutorialStartProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
//    int i;

mfd->lines = 2;
mfd->doubleLines = 2 * mfd->lines;
mfd->alpha = 0.7f;


    return 0;
}



int tutorialRunProc(const FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    PixDim w, h;
    Pixel32 *src, *dst;
	int anz = 0;

    const int lines = mfd->lines;
	const int doubleLines = lines * 2;
    const float alpha = mfd->alpha;

    src = (Pixel32 *)fa->src.data;
    dst = (Pixel32 *)fa->dst.data;

    h = fa->src.h;
    do {
        w = fa->src.w;
		do {
			Pixel32 old_pixel, new_pixel;

			old_pixel = *src++;


			if (anz<lines) {
				*dst++ = old_pixel;
			} else {
				// red = (old_pixel & 0xFF0000)
				// green = (old_pixel>>8) & 0xff
				// blue = old_pixel & 0xff
				new_pixel = 
					          ((unsigned long)((old_pixel & 0xFF0000) * alpha) & 0xFF0000)
							+ ((unsigned long)((old_pixel & 0x00FF00) * alpha) & 0x00FF00)
							+ (unsigned long)((old_pixel & 0x0000FF) * alpha);
							
				*dst++ = new_pixel;
				
			}



		} while(--w);
		if (anz>doubleLines) anz = 0; else anz++;
			
	}while(--h);


    return 0;
}


int tutorialEndProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    //delete[] mfd->lines;
	mfd->lines = NULL;

    return 0;
}

long tutorialParamProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    if (mfd->fExpand)
	    fa->dst.w *= 2;

	fa->dst.pitch = (fa->dst.w*4 + 7) & -8;

    return FILTERPARAM_SWAP_BUFFERS;
}


BOOL CALLBACK tutorialConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

    switch(msg) {
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;
/*
            CheckDlgButton(hdlg, IDC_EXPAND, mfd->fExpand?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_THIRD, mfd->fThird?BST_CHECKED:BST_UNCHECKED);
*/
            return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
				/*
                mfd->fExpand = !!IsDlgButtonChecked(hdlg, IDC_EXPAND);
                mfd->fThird = !!IsDlgButtonChecked(hdlg, IDC_THIRD);
				*/
				mfd->lines = 
                EndDialog(hdlg, 0);
                return TRUE;
            case IDCANCEL:
                EndDialog(hdlg, 1);
                return FALSE;
            }
            break;
    }

    return FALSE;
}

int tutorialConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
    return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER_TUTORIAL), hwnd,
            tutorialConfigDlgProc, (LPARAM)fa->filter_data);
}

void tutorialStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
    const char *modes[2][2]={
        " (normal)",
        " (expand)",
        " (third)",
        " (expand+third)",
    };
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    strcpy(str, modes[mfd->fThird][mfd->fExpand]);
}

void tutorialScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    mfd->fExpand    = !!argv[0].asInt();
    mfd->fThird     = !!argv[1].asInt();
}

bool tutorialFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    _snprintf(buf, buflen, "Config(%d, %d)",
        mfd->fExpand,
        mfd->fThird);

    return true;
}

